//---------------------------------------------------------------------
//  Pass 0 - Environment
//---------------------------------------------------------------------
float4 vViewPosition;
float4x4 matViewProjection;

struct VS_INPUT_0 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
   
};

struct VS_OUTPUT_0 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
   
};

VS_OUTPUT_0 vs_main_0( VS_INPUT_0 Input )
{
   VS_OUTPUT_0 Output;

   Output.Position = mul( float4(Input.Position.xyz + vViewPosition.xyz, 1), matViewProjection );
   Output.Texcoord = normalize(Input.Position.xyz);
   
   return( Output );
   
}

texture EnvironmentMap;
sampler Environment = sampler_state
{
    Texture = <EnvironmentMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

struct PS_INPUT_0 
{
   float3 Texcoord : TEXCOORD0;
   
};

float4 ps_main_0(PS_INPUT_0 Input) : COLOR0
{   
   return( texCUBE(Environment, Input.Texcoord) );
   
}

//---------------------------------------------------------------------
//  Pass 1 : Hatching
//---------------------------------------------------------------------
float4 LightDir;
float4x4 matWorldView;
float4x4 matWorldViewProjection;
float4x4 matWorldViewInverseTranspose;

struct VS_INPUT_1
{
   float4 Pos      : POSITION0;
   float3 Normal   : NORMAL0;
   float2 TexCoord : TEXCOORD0;
};

struct VS_OUTPUT_1
{
   float4 Pos           : POSITION0;
   float2 TexCoord      : TEXCOORD0;
   float3 HatchWeights0 : TEXCOORD1;
   float3 HatchWeights1 : TEXCOORD2;
};

VS_OUTPUT_1 vs_main_1( VS_INPUT_1 Input )
{
   VS_OUTPUT_1 Out; 

   Out.Pos = mul( Input.Pos, matWorldViewProjection );
   Out.TexCoord = Input.TexCoord;

   //float3 posW    = mul( Input.Pos, matViewProjection );
   float3 normalW = normalize(mul( Input.Normal, (float3x3)matWorldViewInverseTranspose ));

   float  diffuse = min(1.0,max(0,dot(normalize(LightDir), normalW)));
   diffuse = diffuse * diffuse;
   diffuse = diffuse * diffuse;

   float  hatchFactor = diffuse * 6.0;
   float3 weight0 = 0.0;
   float3 weight1 = 0.0;

   if (hatchFactor>5.0)
   {
      weight0.x = 1.0;
   }
   else if (hatchFactor>4.0)
   {
      weight0.x = 1.0 - (5.0 - hatchFactor);
      weight0.y = 1.0 - weight0.x;
   }
   else if (hatchFactor>3.0)
   {
      weight0.y = 1.0 - (4.0 - hatchFactor);
      weight0.z = 1.0 - weight0.y;
   }
   else if (hatchFactor>2.0)
   {
      weight0.z = 1.0 - (3.0 - hatchFactor);
      weight1.x = 1.0 - weight0.z;
   }
   else if (hatchFactor>1.0)
   {
      weight1.x = 1.0 - (2.0 - hatchFactor);
      weight1.y = 1.0 - weight1.x;
   }
   else if (hatchFactor>0.0)
   {
      weight1.y = 1.0 - (1.0 - hatchFactor);
      weight1.z = 1.0 - weight1.y;
   }
   Out.HatchWeights0 = weight0;
   Out.HatchWeights1 = weight1;

   return Out;
}

float4 MiscParam;

texture Hatch0_Tex;
sampler Hatch0 = sampler_state
{
    Texture = <Hatch0_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture Hatch1_Tex;
sampler Hatch1 = sampler_state
{
    Texture = <Hatch1_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture Hatch2_Tex;
sampler Hatch2 = sampler_state
{
    Texture = <Hatch2_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture Hatch3_Tex;
sampler Hatch3 = sampler_state
{
    Texture = <Hatch3_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture Hatch4_Tex;
sampler Hatch4 = sampler_state
{
    Texture = <Hatch4_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

texture Hatch5_Tex;
sampler Hatch5 = sampler_state
{
    Texture = <Hatch5_Tex>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

struct PS_INPUT_1
{
   float2 TexCoord      : TEXCOORD0;
   float3 HatchWeights0 : TEXCOORD1;
   float3 HatchWeights1 : TEXCOORD2;
};

struct PS_OUTPUT_1
{
   float4 Color    : COLOR0;
};

PS_OUTPUT_1 ps_main_1( PS_INPUT_1 Input )
{
    PS_OUTPUT_1 Out;
 
    float4 hatchTex0 = tex2D(Hatch0,Input.TexCoord) * Input.HatchWeights0.x;
    float4 hatchTex1 = tex2D(Hatch1,Input.TexCoord) * Input.HatchWeights0.y;
    float4 hatchTex2 = tex2D(Hatch2,Input.TexCoord) * Input.HatchWeights0.z;
    float4 hatchTex3 = tex2D(Hatch3,Input.TexCoord) * Input.HatchWeights1.x;
    float4 hatchTex4 = tex2D(Hatch4,Input.TexCoord) * Input.HatchWeights1.y;
    float4 hatchTex5 = tex2D(Hatch5,Input.TexCoord) * Input.HatchWeights1.z;
    
    float4 hatchColor = hatchTex0 +hatchTex1 +hatchTex2 +hatchTex3 +hatchTex4 +hatchTex5;

    Out.Color = hatchColor;

    return Out;
}

//=================================================================================================================================
//Technique
//=================================================================================================================================
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_2_0 vs_main_0();
        PixelShader  = compile ps_2_0 ps_main_0();
    }
    pass P1
    {
        VertexShader = compile vs_2_0 vs_main_1();
        PixelShader  = compile ps_2_0 ps_main_1();
    }
}