//---------------------------------------------------------------------
//  Pass 0 - Environment
//---------------------------------------------------------------------
float4 vViewPosition;
float4x4 matViewProjection;

struct VS_INPUT_0 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
   
};

struct VS_OUTPUT_0 
{
   float4 Position : POSITION0;
   float3 Texcoord : TEXCOORD0;
   
};

VS_OUTPUT_0 vs_main_0( VS_INPUT_0 Input )
{
   VS_OUTPUT_0 Output;

   Output.Position = mul( float4(Input.Position.xyz + vViewPosition.xyz, 1), matViewProjection );
   Output.Texcoord = normalize(Input.Position.xyz);
   
   return( Output );
   
}

texture EnvironmentMap;
sampler Environment = sampler_state
{
    Texture = <EnvironmentMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

struct PS_INPUT_0 
{
   float3 Texcoord : TEXCOORD0;
   
};

float4 ps_main_0(PS_INPUT_0 Input) : COLOR0
{   
   return( texCUBE(Environment, Input.Texcoord) );
   
}

//---------------------------------------------------------------------
//  Pass 1 : Sparkle
//---------------------------------------------------------------------
float3 fvLightPosition, fvEyePosition;
//float4 vViewPosition;

float4x4 matWorldView, matWorldViewProjection;
//float4x4 matWorldViewInverseTranspose;

float4 I_a, I_d, I_s, k_a, k_d, k_s, k_r;
float VOLUME_NOISE_SCALE = 1;

struct VS_INPUT_1 
{
   float4 Position : POSITION0;
   float3 Tangent  : TANGENT;
   float3 Binormal : BINORMAL;
   float3 Normal   : NORMAL0;
};

struct VS_OUTPUT_1
{
   float4 Position   : POSITION;
   float3 Diffuse    : COLOR0;
   float3 Specular   : COLOR1;
   float3 NoiseCoord : TEXCOORD0;
   float3 Reflection : TEXCOORD1;
   float3 Glossiness : TEXCOORD2;
   float3 HalfVector : TEXCOORD3;
};

VS_OUTPUT_1 vs_main_1( VS_INPUT_1 Input )
{
   VS_OUTPUT_1 Output = (VS_OUTPUT_1)0;

   Output.Position = mul( Input.Position, matWorldViewProjection );

   float3 P = mul(Input.Position, matWorldView);
   float3 V = normalize( mul(vViewPosition - Input.Position, matWorldView) );
   float3 L = normalize(fvLightPosition - P);
   float3 H = normalize(L + V);
   float3 N = normalize( mul(Input.Normal, matWorldView) );
   float3 B = normalize( mul(Input.Binormal, matWorldView) );
   float3 T = normalize( mul(Input.Tangent, matWorldView) );
   float3 R = normalize(2 * dot(N,L) * N - L);
   
   float3 Eye = normalize(vViewPosition - Input.Position);   // local or world space
   //float4 N = normalize( mul(Input.Normal, matWorldView) );
   //float3 Eye = normalize(-Position);   // View Space
   //float3 Reflection = normalize( 2 * dot(N, Eye) * N - Eye );

   float3 Reflection = normalize( 2.0 * dot(Input.Normal, Eye) * Input.Normal - Eye );   
   float3 Refraction = normalize( 0.5 * dot(Input.Normal, Eye) * Input.Normal - Eye );
   //float3 Refraction = normalize( refract(-Eye, Input.Normal, 0.6) );
   float3 G = normalize( 2 * dot(N, V) * N - V );
   float f = pow( 1 - saturate(dot(R, N) ), 1);
   
   Output.Diffuse    = I_a * k_a + I_d * k_d * max(0, dot(N, L));
   Output.Specular   = pow(saturate(dot(R, N)), 4) * k_s;
   Output.Glossiness = f * k_r;
   Output.HalfVector = float3(dot(T, H), dot(B, H), dot(N, H));   // Tangent Space
   Output.Reflection = Refraction; // Reflection or Refraction
   Output.NoiseCoord = Input.Position * VOLUME_NOISE_SCALE;   // 3D Texcoord(u, v, w)
   
   return( Output );
   
}

//float4 k_s;
texture SparkleNoiseMap;
sampler3D SparkleNoise = sampler_state
{
    Texture = <SparkleNoiseMap>;
    MinFilter = LINEAR;
    MagFilter = LINEAR;
    MipFilter = LINEAR;

    AddressU = Clamp;
    AddressV = Clamp;
};

struct PS_INPUT_1
{
   float3 Diffuse    : COLOR0;
   float3 Specular   : COLOR1;
   float3 NoiseCoord : TEXCOORD0;
   float3 Reflection : TEXCOORD1;
   float3 Glossiness : TEXCOORD2;
   float3 HalfVector : TEXCOORD3;
};

float4 ps_main_1(PS_INPUT_1 Input) : COLOR0
{  
   float4 Color = (float4)0;
   float3 Diffuse, Specular, Gloss, Sparkle;
   // Volume noise
   float4 Noise = tex3D(SparkleNoise, Input.NoiseCoord);
   // Noisy diffuse of metal
   Diffuse = Input.Diffuse * Noise.r;
   // Glossy specular of wax
   Specular = Input.Specular;
   Specular *= Specular;
   Specular *= Specular;
   // Glossy reflection of wax
   Gloss = texCUBE(Environment, Input.Reflection) * saturate(Input.Glossiness);
   //Gloss = texCUBE(EnvironmentMap, Input.Reflection);
   // Specular sparkle of flakes
   Sparkle = saturate(dot((saturate(Input.HalfVector) - 0.5) * 2, (Noise.rgb - 0.5) * 2));
   Sparkle *= Sparkle;
   Sparkle *= Sparkle;
   Sparkle *= Sparkle;
   Sparkle *= k_s;
   // Combine the contributions
   Color.rgb = Diffuse + Specular + Gloss + Sparkle + Input.Glossiness;
   Color.w = 1;
 
    return Color;
   //return( texCUBE(Texture0, Input.Texcoord) );
   
}

//=================================================================================================================================
//Technique
//=================================================================================================================================
technique TShader
{
    pass P0
    {
        VertexShader = compile vs_2_0 vs_main_0();
        PixelShader  = compile ps_2_0 ps_main_0();
    }
    pass P1
    {
        VertexShader = compile vs_2_0 vs_main_1();
        PixelShader  = compile ps_2_0 ps_main_1();
    }
}